"use strict";
(self["webpackChunkserialhub"] = self["webpackChunkserialhub"] || []).push([["lib_index_js"],{

/***/ "./lib/index.js":
/*!**********************!*\
  !*** ./lib/index.js ***!
  \**********************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "default": () => (__WEBPACK_DEFAULT_EXPORT__),
/* harmony export */   "MODULE_NAME": () => (/* reexport safe */ _version__WEBPACK_IMPORTED_MODULE_0__.MODULE_NAME),
/* harmony export */   "MODULE_VERSION": () => (/* reexport safe */ _version__WEBPACK_IMPORTED_MODULE_0__.MODULE_VERSION),
/* harmony export */   "SerialHubModel": () => (/* reexport safe */ _widget__WEBPACK_IMPORTED_MODULE_1__.SerialHubModel),
/* harmony export */   "SerialHubView": () => (/* reexport safe */ _widget__WEBPACK_IMPORTED_MODULE_1__.SerialHubView),
/* harmony export */   "SerialHubPort": () => (/* reexport safe */ _serialhubport__WEBPACK_IMPORTED_MODULE_2__.SerialHubPort)
/* harmony export */ });
/* harmony import */ var _version__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./version */ "./lib/version.js");
/* harmony import */ var _widget__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./widget */ "./lib/widget.js");
/* harmony import */ var _serialhubport__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./serialhubport */ "./lib/serialhubport.js");
// Copyright (c) cdr4eelz
// Distributed under the terms of the Modified BSD License.
/**
 * Initialization data for the serialhub extension.
 */
const plugin = {
    id: 'serialhub:plugin',
    autoStart: true,
    activate: (app) => {
        console.log('JupyterLab extension serialhub is NOW activated!');
    }
};
/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (plugin);




//# sourceMappingURL=index.js.map

/***/ })

}]);
//# sourceMappingURL=lib_index_js.860159e25d6f0b4ab178.js.map