#!/usr/bin/env python
# coding: utf-8

# Copyright (c) cdr4eelz.
# Distributed under the terms of the Modified BSD License.

"""
Information about the frontend package of the widgets.
"""

module_name = "serialhub" #pylint: disable=invalid-name
module_version = "^0.0.35"  # pylint: disable=invalid-name
